# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------------
# Line_Cross_20150722.py
# Created on: 2015-07-22 12:12:41.00000
#   (generated by ArcGIS/ModelBuilder)
# Description: This tool creates polygons where more than 3 lines connect.
#  Created by Jean M. Olson
# ---------------------------------------------------------------------------

# Import arcpy module
import arcpy

# Set the workspace environment
arcpy.env.overwriteOutput = True

# Script arguments
# Input data

Input_Network = arcpy.GetParameterAsText(0)
Workspace = arcpy.GetParameterAsText(1)
Output_Polygon = arcpy.GetParameterAsText(2)
Scratch = arcpy.GetParameterAsText(3)

def main(Input_Network,Workspace,Output_Polygon,Scratch):

    # Local variables:
    Network_Points = Scratch + "\\Network_Points"
    Start_Pt_Buffer = Scratch + "\\Start_Pt_Buffer"
    Buffer_Clip = Scratch + "\\Buffer_Clip"
    Clip_Mid_Pts = Scratch + "\\Clip_Mid_Pts"
    Buffer_Points = Scratch + "\\Buffer_Points"
    Buffer_Select = Scratch + "\\Buffer_Select"

    # Process: Feature Vertices To Points - Create the start points of all the lines.
    arcpy.Intersect_analysis(Input_Network, Network_Points,output_type="POINT")

    # Process: Buffer - Buffer the start points of all the the lines.
    arcpy.Buffer_analysis(Network_Points, Start_Pt_Buffer, "1 Meters", "FULL", "ROUND", "NONE", "")

    # Process: Clip - Clip the original network by the point buffer.
    arcpy.Clip_analysis(Input_Network, Start_Pt_Buffer, Buffer_Clip, "")

    # Process: Feature Vertices To Points - Create the mid-points of the clipped lines.
    arcpy.FeatureVerticesToPoints_management(Buffer_Clip, Clip_Mid_Pts, "MID")

    # Process: Spatial Join - Join the mid-points back to the buffer polygons.
    arcpy.SpatialJoin_analysis(Start_Pt_Buffer, Clip_Mid_Pts, Buffer_Points, "JOIN_ONE_TO_ONE", "KEEP_ALL", "INTERSECT", "", "")

    # Process: Select - Select only those polygons where the join count is more than 3 - more than 3 lines intersect at a point.
    arcpy.Select_analysis(Buffer_Points, Buffer_Select, "\"Join_Count\" >3")

    # Process: Dissolve - Dissolve the buffer polygons in case of overlap.
    arcpy.Dissolve_management(Buffer_Select, Workspace + "\\" + Output_Polygon, "", "", "SINGLE_PART", "DISSOLVE_LINES")

